{
CallerId version 1.0
Johan W. Strikkers - j.w.strikkers@hccnet.nl
Take it as it is!

properties:
Active
Installed
Manufacturer
Version.CapiMajor 
       .CapiMinor 
       .ManufacturerMajor
       .ManufacturerMinor
Msn: No number -> react on all incoming calls
     numbers   -> react only on incoming for these Msn-numbers
UnknownText: Text used when callernumber is unknown.

Event:
OnRing
}

unit CallerId;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, CALLERDEFS;

type
  TRingEvent = procedure (Caller ,Receiver: string) of object;

type
  TCallerId = class(TComponent)
  private
    { Private declarations }
    AppId: Dword;
    CapiBuffer: Dword;
    FTimer: TTimer;
    FActive: boolean;
    FInstalled: boolean;
    FManufacturer: string;
    FVersion: TVersion; 
    FOnRing: TRingEvent;
    FMsn: TStrings;
    FUnknownText: string;
    procedure SetUnknownText(val: string);
    procedure SetActive(val: boolean);
    procedure CheckCalls(Sender: TObject);
    function GetVersion: TVersion;
    procedure GetManufacturer;
    procedure SetMsn(Value : TStrings);
  protected        
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Active: Boolean read FActive write SetActive;
    property Installed: boolean read FInstalled;
    property Manufacturer: string read FManufacturer;
    property Version: Tversion read FVersion;
  published
    { Published declarations }
    property Msn : TStrings read FMsn write SetMsn;
    property UnknownText: string read FUnknownText write SetUnknownText;
    property OnRing: TRingEvent read FOnRing write FOnRing;
  end;

  procedure Register;
  function CAPI_INSTALLED: DWORD; stdcall;
  function CAPI_GET_MANUFACTURER(SzBuffer: pointer): DWORD; stdcall;
  function CAPI_GET_VERSION(pCAPIMajor, pCAPIMinor, 
                            pManufacturerMajor,pManufacturerMinor: pointer): DWord; stdcall;
  function CAPI_REGISTER(const a,b,c,d: Dword; p:pointer): Dword; stdcall;
  function CAPI_RELEASE(const AppId:Dword): Dword; stdcall;
  function CAPI_PUT_MESSAGE(const AppId: Dword; Req: pointer): Dword ; stdcall; 
  function CAPI_GET_MESSAGE(Const AppID: Dword; lpCapiBuffer: pointer): Dword; stdcall;
  procedure RtlMoveMemory(MessageHeader: Pointer; const lpCapiBuffer: DWORD; const HeaderLength: integer); stdcall;


implementation
  function CAPI_INSTALLED: DWORD; stdcall; external 'Capi2032.dll';
  function CAPI_GET_MANUFACTURER(SzBuffer: pointer): DWORD; stdcall; external 'Capi2032.dll';
  function CAPI_GET_VERSION(pCAPIMajor, pCAPIMinor, pManufacturerMajor,
                            pManufacturerMinor: pointer): DWord; stdcall; external 'Capi2032.dll';  
  function CAPI_REGISTER(const a,b,c,d: Dword; p:pointer): Dword; stdcall; external 'Capi2032.dll';
  function CAPI_RELEASE(const AppId: Dword): Dword; stdcall; external 'Capi2032.dll';
  function CAPI_PUT_MESSAGE(const AppId: Dword; Req: Pointer): Dword ; stdcall;  external 'Capi2032.dll';
  function CAPI_GET_MESSAGE(Const AppID: Dword; lpCapiBuffer: pointer): Dword; stdcall; external 'Capi2032.dll';

  procedure RtlMoveMemory(MessageHeader: Pointer; const lpCapiBuffer: DWORD; const HeaderLength: integer); stdcall; external 'Kernel32.dll';


constructor TCallerId.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FMsn := TStringList.Create;  
  TstringList(FMsn).Sorted := True;
  FUnknownText := 'Unknown'; 
  FInstalled := False;
  if not (CAPI_INSTALLED = 0) then begin
    MessageBox(0,Pchar('Capi not installed'), PChar('Not installed'),mb_ok);
    exit;
  end;  
  FInstalled := True;
  AppId := 0;
  FActive := False;
  FTimer := TTimer.Create(self);
  FTimer.Enabled := False;
  FTimer.Interval := 400;
  FTimer.OnTimer := CheckCalls;
  GetManufacturer;         
  GetVersion; 
end;

procedure TCallerId.SetUnknownText(val: string);
begin
  if val <> FUnknownText then FUnknownText := val;
end;

procedure TCallerId.SetMsn(Value : TStrings);
begin                          
  if Value.Count = 0 then exit;
  FMsn.Assign(Value);          
end;                           

procedure TCallerId.GetManufacturer;
var Manufacturer: array[0..63] of char;
begin
  if CAPI_GET_MANUFACTURER(@Manufacturer) = 0 then
     FManufacturer := Manufacturer else FManufacturer := 'No manufacturer';
end;  

function TCallerId.GetVersion: TVersion;
var CapiMajor, CapiMinor, ManufacturerMajor, ManufacturerMinor: Dword;
begin
  if CAPI_GET_VERSION(@CAPIMajor, @CAPIMinor, 
     @ManufacturerMajor,@ManufacturerMinor) = 0 then begin
     FVersion.CapiMajor := CapiMajor;
     FVersion.CapiMinor := CapiMinor;
     FVersion.ManufacturerMajor := ManufacturerMajor;
     FVersion.ManufacturerMinor := ManufacturerMinor;
   end else begin
     FVersion.CapiMajor := 0;
     FVersion.CapiMinor := 0;
     FVersion.ManufacturerMajor := 0;
     FVersion.ManufacturerMinor := 0;
   end;
end;

destructor TCallerId.Destroy;
begin
  if FActive then begin
    FTimer.Enabled := False;
    CAPI_RELEASE(AppId);
  end;
  FTimer.Free;  
  inherited Destroy;
end;

procedure TCallerId.SetActive(Val:boolean);
var Error: boolean;
    ListenReq: TListenREQ;
begin
  if (Val = FActive) or (not Finstalled) then exit;
  Error := False;
  if FActive then begin
    CAPI_RELEASE(AppId);
    AppId := 0;
  end else begin
    if CAPI_REGISTER(3072,2,7,2048,@AppId) <> 0 then Error := True;
    with ListenREQ do begin
      Lengte          := SizeOf(ListenREQ);
      ApplId          := appId;
      Command         := $5;
      SubCommand      := $80;
      Message_number  := 1;
      Controller      := 1;
      InfoMask        := InfoMask_ALL;
      CIPMask         := CIPMASK_ALL;
      CIPMas2         := CIPValue_ALL;
      CallingParty    := $0;
      CallingPartySub := $0;
    end;                    
    if CAPI_PUT_MESSAGE(AppId, @ListenREQ) <> 0 then begin
      if error then begin
        CAPI_RELEASE(AppId); 
        AppId := 0;
      end;  
      Error := True;
    end;
  end;
  if not Error then FActive := Val else FActive := False;
  FTimer.Enabled := FActive;
end;

procedure TCallerId.CheckCalls;
var x: Dword;
    MessageHeader: TMessageHeader;
    ConnectInd: TConnectInd;
    FCaller: string[11];
    FReceiver: string[11];
    FCallerNumber: string;
    msg : string[255];
    l1,l2: integer;
    AlertReq: TAlertREQ;
    ConnectResp: TConnectResp;
    InfoInd: TInfoInd;
    InfoResp: TInfoResp;
    DisConnectInd: TDisconnectInd;
    DisConnectResp: TDisconnectResp;
    n,c: integer;
begin
  x := CAPI_GET_MESSAGE(appId,@CapiBuffer);
  if x = 0 then begin
    FTimer.Enabled := False;
    RtlMoveMemory(@MessageHeader, CapiBuffer, HeaderLength); 
    Case  MessageHeader.Command of
      CAPI_CONNECT: begin
        If MessageHeader.SubCommand = CAPI_IND Then
           RtlMoveMemory(@ConnectInd, CapiBuffer + HeaderLength, 
                         MessageHeader.length - HeaderLength);
           msg := ConnectInd.Buffer;
           l1 := ord(ConnectInd.Buffer[0]);
           FReceiver := Copy(ConnectInd.Buffer,3,l1-1);
           if FReceiver[1] <> '0' then FReceiver := '0' + FReceiver;
           l2 := ord(ConnectInd.Buffer[l1 + 1]);
           if l2 > 0 Then //begin
             if l2 > 2 Then begin
                FCaller := Copy(ConnectInd.Buffer,l1+5,l2-2);
                if FCaller[1] <> '0' then FCaller := '0' + FCaller;
                FCallerNumber := FCaller;
             end else FCallerNumber := FUnknownText;
           if (Assigned(Onring)) and  ((FMsn.IndexOf(FReceiver) > -1) or 
              (FMsn.Count = 0)) then OnRing(FCallerNumber,FReceiver);
           with AlertReq do begin
             length         := SIZEOF(AlertReq);
             ApplId         := AppId;
             Command        := CAPI_ALERT;
             SubCommand     := CAPI_REQ;
             Message_Number := $0;
             PLCI           := ConnectInd.PLCI;
             BCI            := 0;
             kpf            := 0;
             uud            := 0;
             fda            := 0;
           end;
           x := CAPI_PUT_MESSAGE(AppId,@AlertReq);
           with ConnectResp do begin
             Length              := SizeOf(ConnectResp);
             Applid              := Appid;
             Command             := CAPI_CONNECT;
             SubCommand          := CAPI_RESP;
             Message_Number      := 0;
             PLCI                := ConnectInd.PLCI;
             REJECT              := CALL_RESP_IGNORE;
             BProtocol           := $0;
             ConnectedNumber     := $0;
             ConnectedSubaddress := $0;
             LLC                 := $0;
             AdditionalInfo      := $0;
           end;
           x := CAPI_PUT_MESSAGE(AppId, @ConnectResp);
      end;
      CAPI_INFO: begin
        If MessageHeader.SubCommand = CAPI_IND then
          RtlMoveMemory(@InfoInd, CapiBuffer + Headerlength, MessageHeader.length - Headerlength);
  //wat ik met InfoInd.InfoElement moet weet ik niet
          With InfoResp do begin
            length         := Sizeof(InfoResp);
            ApplId         := AppId;
            Command        := CAPI_INFO;
            SubCommand     := CAPI_RESP;
            Message_Number := $0;
            PLCI           := ConnectInd.PLCI;
          end;
          x := CAPI_PUT_MESSAGE(AppId, @InfoResp);
      end;
      CAPI_DISCONNECT: begin
        if MessageHeader.SubCommand = CAPI_IND then
          RtlMoveMemory(@DisConnectInd, CapiBuffer + Headerlength, MessageHeader.length - Headerlength);
  // hier moet decode_info
          with DisConnectResp do begin
            Length         := SizeOf(DisConnectResp);
            ApplId         := AppId;
            Command        := CAPI_DISCONNECT;
            SubCommand     := CAPI_RESP;
            Message_Number := $0;
            PLCI           := DisConnectInd.PLCI;
          end;
          x := CAPI_PUT_MESSAGE(AppId, @DisConnectResp);
      end;    
    end;
    FTimer.Enabled := True;
  end;  
end;

procedure Register;
begin
  RegisterComponents('Samples', [TCallerId]);
end;

end.
